/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef __PLUGIN_H__
#define __PLUGIN_H__

#include <map>
#include <string>

/* Xlib/Xt stuff */
#include <X11/Xlib.h>
#include <X11/cursorfont.h>

#include <prtypes.h>
#include <limits.h>

#include "npapi.h"
#include "npruntime.h"

#include "pluginbase.h"

#define DWORD uint32_t
#define BOOL bool
#include "controller.h"

class nsPluginInstance : public nsPluginInstanceBase
{
public:
  nsPluginInstance(NPP aInstance);
  virtual ~nsPluginInstance();

  NPBool init(NPWindow* aWindow);
  void shut();
  NPBool isInitialized();

  NPError	GetValue(NPPVariable variable, void *value);
  NPError SetWindow(NPWindow* aWindow);

  // locals
  void Connect();
  void disconnect();
  void show();
  void ConnectedStatus(PRInt32 *_retval);
  void SetLanguageStrings(const char * aSection, const char * aLanguage);
  void SetUsbFilter(const char * aUsbFilter);

  /* attribute ing Host; */
  void GetHostIP(char * *aHosIPt);
  void SetHostIP(const char * aHostIP);

  /* attribute ing Port; */
  void GetPort(char * *aPort);
  void SetPort(const char * aPort);

  /* attribute ing Password; */
  void GetPassword(char * *aPassword);
  void SetPassword(const char * aPassword);

  /* attribute ing SecurePort; */
  void GetSecurePort(char * *aSecurePort);
  void SetSecurePort(const char * aSecurePort);

  /* attribute ing Port; */
  void GetCipherSuite(char * *aCipherSuite);
  void SetCipherSuite(const char * aCipherSuite);

  /* attribute ing Port; */
  void GetSSLChannels(char * *aSSLChannels);
  void SetSSLChannels(const char * aSSLChannels);

  /* attribute ing TrustStore; */
  void GetTrustStore(char * *aTrustStore);
  void SetTrustStore(const char * aTrustStore);

  /* attribute ing HostSubject; */
  void GetHostSubject(char * *aHostSubject);
  void SetHostSubject(const char * aHostSubject);

  /* attribute ing FullScreen; */
  void GetFullScreen(PRBool *aFullScreen);
  void SetFullScreen(PRBool aFullScreen);

  /* attribute ing Port; */
  void GetTitle(char * *aTitle);
  void SetTitle(const char * aTitle);

  /* attribute ing Port; */
  void GetDynamicMenu(char * *aDynamicMenu);
  void SetDynamicMenu(const char * aDynamicMenu);

  /* attribute ing Port; */
  void GetNumberOfMonitors(char * *aNumberOfMonitors);
  void SetNumberOfMonitors(const char * aNumberOfMonitors);

  /* attribute ing AdminConsole; */
  void GetAdminConsole(PRBool *aAdminConsole);
  void SetAdminConsole(PRBool aAdminConsole);

  /* attribute ing GuestHostName; */
  void GetGuestHostName(char * *aGuestHostName);
  void SetGuestHostName(const char * aGuestHostName);

  /* attribute ing HotKeys; */
  void GetHotKeys(char * *aHotKeys);
  void SetHotKeys(const char * aHotKeys);

  /* attribute ing NoTaskMgrExecution; */
  void GetNoTaskMgrExecution(PRBool *aNoTaskMgrExecution);
  void SetNoTaskMgrExecution(PRBool aNoTaskMgrExecution);

  /* attribute ing SendCtrlAltdelete; */
  void GetSendCtrlAltdelete(PRBool *aSendCtrlAltdelete);
  void SetSendCtrlAltdelete(PRBool aSendCtrlAltdelete);

  /* attribute unsigned short UsbListenPort; */
  void GetUsbListenPort(unsigned short *aUsbPort);
  void SetUsbListenPort(unsigned short aUsbPort);

  /* attribute boolean UsbAutoShare; */
  void GetUsbAutoShare(PRBool *aUsbAutoShare);
  void SetUsbAutoShare(PRBool aUsbAutoShare);

  NPObject* getScriptablePeer();

  pid_t m_child_pid;
  PRInt32 m_connected_status;
  SpiceController *m_external_controller;
  
private:  
  static void* UsbCtrlWatchDog(void *opaque);
  void ExecuteUsbCtrl();
  
private:
  NPP mInstance;
  NPBool mInitialized;

  NPWindow *mWindow;
  Visual* mVisual;
  Colormap mColormap;
  unsigned int mDepth;
  XFontStruct *mFontInfo;

    char m_strHost[256];
    char m_strPort[256];
    char m_strPassword[256];
    PRBool m_fIPC;
    char m_HostIP[256];
    char m_Port[256];
    char m_Password[256];
    char m_SecurePort[256];
    char m_CipherSuite[256];
    char m_SSLChannels[256];
    char m_TrustStore[4096];
    wchar_t m_HostSubject[256];
    PRBool m_FullScreen;
    PRBool m_AdminConsole;
    char m_Title[256];
    char m_DynamicMenu[256];
    char m_NumberOfMonitors[256];
    char m_GuestHostName[256];
    char m_HotKeys[256];
    PRBool m_NoTaskMgrExecution;
    PRBool m_SendCtrlAltdelete;
    unsigned short m_UsbPort;
    std::string m_UsbFilter;
    PRBool m_UsbAutoShare;
    std::map<std::string, std::string> m_Language;
    
  pid_t m_UsbCtrlPid;
  
  NPObject* mScriptablePeer;
 char mHomeDir[PATH_MAX];

public:
  char mString[128];
};

#endif // __PLUGIN_H__
